// -*- mode:c++ -*-
//
// Header file Generator.hh
//
// Copyright (c) 2001 Raoul M. Gough
//
// History
// =======
// 2001/ 7/29	rmg	File creation
//

#ifndef Generator_included
#define Generator_included

#include <iosfwd>
#include "SMLState.hh"
#include "LookaheadContext.hh"

class SMLLexer;

class Generator
{
public:
  Generator (SMLLexer&, std::istream& decor, std::ostream& out);

public:
  ~Generator ();

public:
  void generate (void);

private:
  void closeTag (char t);

private:
  void popStack (void);

private:
  void unwind (void);

private:
  void pushContext (void);

private:
  void popContext (void);

private:
  void checkContextChange (SMLState);

private:
  void reachState (SMLState);

private:
  struct PushTag {
    const char *tag;
    void (SMLState::*setter) (void);
    LookaheadContext::PositionStack *needed;
    LookaheadContext::PositionContainer::const_reverse_iterator neededPos;

    PushTag (const char *t, void (SMLState::*s) (void))
      : tag (t)
      , setter (s)
      , needed (0)
      , neededPos ()
    {
    }

    PushTag (const char *t
	     , void (SMLState::*s) (void)
	     , LookaheadContext::PositionStack&);
  };

  struct PushTagList {
    bool setPlain;
    std::vector<PushTag> ptl;
    int size;
    int colour;
    int cost;

    PushTagList (void)
      : setPlain (false)
      , ptl ()
      , size (-1)
      , colour (-1)
      , cost (0)
    {
    }

    PushTagList (SMLState before, SMLState after, LookaheadContext&);
  };

  void sortList (PushTagList&);
  void runList (const PushTagList&);
  bool stackSearch (SMLState, int);

private:
  void openTag (const char*);

private:
  void openTag (const PushTag&);

private:
  struct StackContext {
    char tag;
    SMLState::Packed state;
    LookaheadContext::PositionStack *needed;
    LookaheadContext::PositionContainer::const_reverse_iterator neededPos;

    StackContext (void)
      : tag (0)
      , state ()
      , needed (0)
      , neededPos ()
    {
    }

    StackContext (char t, SMLState::Packed s)
      : tag (t)
      , state (s)
      , needed (0)
      , neededPos ()
    {
    }

    StackContext (const PushTag& push, SMLState::Packed s)
      : tag (*push.tag)
      , state (s)
      , needed (push.needed)
      , neededPos (push.neededPos)
    {
    }
  };

  typedef vector<StackContext> GeneratorStack;

  SMLLexer& mLex;
  std::istream& mDecor;
  std::ostream& mOut;

  GeneratorStack mStack;
  SMLState mState;

  std::vector<int>::const_iterator mLookaheadIndexIter;
  LookaheadContext *mContextp;

  bool mColourOutput;
  bool mSizeOutput;

  int mInputPos;
};

#endif // Generator_included
