// -*- mode:c++ -*-
//
// Module lextest.cc
//
// Copyright (c) 2001 Raoul M. Gough
//
// History
// =======
// 2001/ 7/27	rmg	File creation
//

#include <iostream>
#include <string>
#include <algorithm>
#include <fstream>
#include <cstdlib>
#include "SMLLexer.hh"
#include "Generator.hh"
#include "utils.hh"

using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::copy;
using std::ostream_iterator;
using std::fstream;
using std::ios;

static void run (const char *, const char *, const char *);

int main (void)
{
  try
    {
      run ("/tmp/lextest.orig", "/tmp/lextest.out", "/tmp/lextest.opt");
#if !DEBUG
        system ("rm /tmp/lextest.orig /tmp/lextest.out /tmp/lextest.opt");
#endif
    }

  catch (...)
    {
      cerr << "Unknown exception. Aborting without cleanup..." << endl;
    }
}


static void run (const char *origName
		 , const char *decorName
		 , const char *optName)
{
  fstream original (origName
		    , ios::out
		    | ios::in
		    | ios::trunc);

  fstream decor (decorName
		 , ios::out
		 | ios::in
		 | ios::trunc
		 | ios::binary);

  SMLLexer lex (decor, &cin, &original);

  try
    {
      lex.yylex();
    }

  catch (const StackEmpty& err)
    {
      cerr << "Stack underflow at ";
      lex.dumpPosition (cerr);
      cerr << endl;
    }

  catch (const TagMismatch& mis)
    {
      cerr << "Tag mismatch at ";
      lex.dumpPosition (cerr);
      cerr << endl;
      cerr << "Expecting " << mis.first << " but got " << mis.second << endl;
    }

  catch (const string& err)
    {
      cerr << err << " at ";
      lex.dumpPosition (cerr);
      cerr << endl;
    }

#if DEBUG
  cerr << "Lookahead dump: " << endl << lex.getLookahead();
  cerr << "Lookahead order: ";
  copy (lex.lookaheadIndexBegin(), lex.lookaheadIndexEnd(),
	ostream_iterator<int>(cerr, ", "));
  cerr << endl;
#endif


  decor.seekg(0);
  decor.clear();

  fstream optimised (optName
		     , ios::out
		     | ios::in
		     | ios::trunc);

  Generator g (lex, decor, optimised);

  g.generate ();

  if (original.tellp() < optimised.tellp())
    {
      original.seekg(0);
      original.clear();
      cout << original.rdbuf();
    }

  else
    {
      optimised.seekg(0);
      optimised.clear();
      cout << optimised.rdbuf();
    }
}
